"""Adapts our charts to the twisted framework.


Here's what you can do with the QuickChart resource...

http://localhost:8000/quickchart?
  qc.titleText=Hello &
  qc.seriesNames=['fund','index']

To display a chart on a certain URL, use this:
from rlextra.graphics.webcharts.twistedcharts import TwistedQuickChartResource

root.putChild('quickchart', TwistedQuickChartResource())

Better still would be a chart directory which scanned for them.

"""


from twisted.application import internet, service
from twisted.web import resource, static, server
import os, sys

from reportlab.lib.utils import recursiveImport, recursiveSetAttr, recursiveGetAttr
from reportlab.graphics.renderPM import drawToString
from reportlab.graphics.shapes import Drawing
from rlextra.graphics.quickchart import QuickChart


class TwistedDrawingResource(resource.Resource):
    """Wraps a ReportLab drawing instance, constructed normally,
    and adapts it for use in Twisted as a resource object.

    """    

    def __init__(self, rlDrawing):
        self.rlDrawing = rlDrawing

    def render_GET(self, request):

        params = {}
        for arg in request.args:
            value = request.args[arg][0]
            # our app strips out empty params, theirs does not.
            # this means we would look up a lot of funds called ''
            if value == '':
                continue

            # NEED SAFE EVAL!  Names assumed to be strings
            try:
                value = eval(value)
            except NameError:
                pass  #it's a string
            recursiveSetAttr(self.rlDrawing, arg, value)

        binaryData = drawToString(self.rlDrawing)        
        request.setHeader("content-type", 'image/gif')
        return binaryData

    def render_POST(self, request):
        return self.render_GET(request)



class TwistedQuickChartResource(TwistedDrawingResource):
    def __init__(self):
        d = Drawing(height=270, width=400)
        d.add(QuickChart(), name='qc')
        self.rlDrawing = d

## chart directory based on this
##class ResourceScriptDirectory(resource.Resource):
##    def __init__(self, pathname, registry=None):
##        resource.Resource.__init__(self)
##        self.path = pathname
##        self.registry = registry or static.Registry()
##
##    def getChild(self, path, request):
##        fn = os.path.join(self.path, path)
##
##        if os.path.isdir(fn):
##            return ResourceScriptDirectory(fn, self.registry)
##        if os.path.exists(fn):
##            return ResourceScript(fn, self.registry)
##        return error.NoResource()
##
##    def render(self, request):
##        return error.NoResource().render(request)
