"""Adapts our WebApps to the twisted framework.

Our WebApp class was designed to be portable.  The first proof
of this was that it took about one hour to get running under
Twisted rather than CGI (making a full PDF document off a multi
step wizard), using the old .rpy files and mktap command.

Advice off the Twisted list is that the more modern way to
do it is with .tac files and a special kind of resource.


The file below worked for me when saved as 'foo.tac'
and executed with
  twistd.py -noy foo.tac

After doing this I could (a) browse static stuff,
(b) browse the kfdimages virtual directory, and
(c) execute you-know-what with /kfdserve.app?action=test_menu


==============sample tac file for kfdserver========
# my first tac

from twisted.application import internet, service
from twisted.web import resource, static, server

import os, sys
from rlextra.ers.webapp import WebApp, WebRequest, WebResponse
from rlextra.ers.twistedwebapp import TwistedWebAppResource

# Construct the application
application = service.Application("web")

root = static.File('c:/program files/apache group/apache2/htdocs')
root.putChild('kfdimages', static.File('c:/code/fidelity/kfd/kfdserver/kfdimages'))



#now set up my first app - note our current directory assumptions
#are dangerous, would be better if each resource hopped into its
#current directory when rendered
os.chdir('c:\\code\\fidelity\\kfd\\kfdserver')
sys.path.insert(0, os.getcwd())
import kfdapp
app = kfdapp.KfdApp(OUTDIR='c:/program files/apache group/apache2/htdocs/pdfout',
                    HTTP_OUTDIR='/pdfout')
kfdres = TwistedWebAppResource(app)


root.putChild('kfdserve.app', kfdres)

site = server.Site(root)

sc = service.IServiceCollection(application)
i = internet.TCPServer(8000, site)
i.setServiceParent(sc)
======================end tac file==================================
"""


from twisted.application import internet, service
from twisted.web import resource, static, server
import os, sys
from rlextra.ers.webapp import WebApp, WebRequest, WebResponse



class TwistedWebAppResource(resource.Resource):
    """Wraps a ReportLab webapp instance, constructed normally,
    and adapts it for use in Twisted as a resource object.

    References to the ReportLab classes WebApp, WebRequest
    and WebResponse are prefixed with rl (e.g. rlApp) to make
    clear whose framework the bits come from.  Other stuff in
    this module uses twisted conventions.

    """    

    def __init__(self, rlApp):
        #we instantiate one and keep it.
        #bear in mind there can only be one current working directory
        #in the long-running-process world
        self.rlApp = rlApp

    def render_GET(self, request):

        rlReq = WebRequest()
        rlReq.url = request.path
        for arg in request.args:
            value = request.args[arg][0]
            # our app strips out empty params, theirs does not.
            # this means we would look up a lot of funds called ''
            if value <> '':
                rlReq.params[arg] = value

        #put in some specials found in our framework
        rlReq.params['CGI_SCRIPT'] = rlReq.params['SCRIPT_NAME'] = rlReq.url

        #workaround for some crap in kfdserve which warns on
        #repeated calls - should be removed.
        rlReq.params['SUPPRESS_CALL_COUNT'] = 1
        

        rlResponse = self.rlApp.handleRequest(rlReq)

        #ensure content type is OK
        ctype = rlResponse.getContentType()
        request.setHeader("content-type", ctype)

        return rlResponse.getContent()

    def render_POST(self, request):
        return self.render_GET(request)



